#ifdef _cplusplus
extern "C" {
#endif
#include "hsp.h"


/* Function:  new_TopScoreManager(length)
 *
 * Descrip:    Makes a new topscore manager of a specific length
 *
 *
 * Arg:        length [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
# line 80 "hsp.dy"
TopScoreManager * new_TopScoreManager(int length)
{


  TopScoreManager * tsm;

  tsm = TopScoreManager_alloc();
  tsm->score = calloc(length,sizeof(int));
  tsm->current_pos = 0;
  tsm->length = length;

  tsm->worst_score = 0;
  tsm->worst_position = -1;

  return tsm;
}

/* Function:  add_score_TopScoreManager(tsm,score)
 *
 * Descrip:    Adds a top score
 *
 *
 * Arg:          tsm [UNKN ] Undocumented argument [TopScoreManager *]
 * Arg:        score [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
# line 100 "hsp.dy"
boolean add_score_TopScoreManager(TopScoreManager * tsm,int score)
{
  int i;
  int temp_score;
  int temp_pos;

/*  fprintf(stderr,"Looking at %d\n",score);*/

  if( tsm->current_pos < tsm->length ) {
/*    fprintf(stderr,"Adding as not enough...\n"); */
    tsm->score[tsm->current_pos++] = score;
    return TRUE;
  }


  if( tsm->worst_position == -1 ) {
    /* fprintf(stderr,"Recalculating...\n"); */
    /* need to recalculate top score */
    for(i=1,temp_score = tsm->score[0],temp_pos = 0;i<tsm->length;i++) {
      if( temp_score > tsm->score[i] ) {
	temp_score = tsm->score[i];
	temp_pos = i;
      }
    }
    tsm->worst_score = temp_score;
    tsm->worst_position = temp_pos;
  }

  if( score < tsm->worst_score ) {
/*    fprintf(stderr,"Ignoring... %d vs %d\n",score,tsm->worst_score); */
    return FALSE;
  }

/*  fprintf(stderr,"Resetting...\n"); */
  tsm->score[tsm->worst_position] = score;
  tsm->worst_position = -1;

  return TRUE;
}


/* Function:  truncated_LinearHSPmanager(lm,max_size,min_score,width,tail)
 *
 * Descrip:    Makes a truncated linear set 
 *
 *
 * Arg:               lm [UNKN ] Undocumented argument [LinearHSPmanager *]
 * Arg:         max_size [UNKN ] Undocumented argument [int]
 * Arg:        min_score [UNKN ] Undocumented argument [int]
 * Arg:            width [UNKN ] Undocumented argument [int]
 * Arg:             tail [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
# line 144 "hsp.dy"
LinearHSPmanager * truncated_LinearHSPmanager(LinearHSPmanager * lm,int max_size,int min_score,int width,int tail)
{
  
  LinearHSPmanager * out;
  int i,j;
  int worst_score = UNFEASIBLY_LARGE_SCORE;
  int worst_j = -1;
  HSPset ** worst;
  HSPset * trial;

  assert(max_size > 0);

  out = LinearHSPmanager_alloc_len(max_size+1);
  out->mat = hard_link_CompMat(lm->mat);

  sort_LinearHSPmanager(lm,compare_HSPset_score);

  for(i=0;i<lm->len;i++) {
    /* we chew up into max_size positions */
    if( out->len < max_size ) {
      trial = new_consistent_HSPset(lm->set[i],min_score,width,tail);
      if( trial->len == 0 ) {
	continue;
      }

      add_LinearHSPmanager(out,trial);
    } else {
          /* if have not found worst, scan to find worst */
      if( worst_score == UNFEASIBLY_LARGE_SCORE ) {
	for(j=0;j<out->len;j++) {
	  if( out->set[j]->score < worst_score ) {
	    worst_score = out->set[j]->score;
	    worst = out->set+j;
	    worst_j = j;
	  }
	}
      }
      /* worst_score is now the score */
      if( lm->set[i]->score < worst_score ) {
	break; /* otta here - we don't need to look at any more! */
      }
      trial = new_consistent_HSPset(lm->set[i],min_score,width,tail);
      if( trial->len == 0 ) {
	continue;
      }


      if( trial->score > worst_score ) {
	free_HSPset(out->set[worst_j]);
	out->set[worst_j] = trial;
	worst_score = UNFEASIBLY_LARGE_SCORE;
      } else {
	free_HSPset(trial);
      }
    }
  }

  return out;

}

/* Function:  new_consistent_HSPset(set,min_score,width,tail)
 *
 * Descrip:    Makes an HSP set via heuristics
 *             to deal with low complexity regions
 *
 *
 * Arg:              set [UNKN ] Undocumented argument [HSPset *]
 * Arg:        min_score [UNKN ] Undocumented argument [int]
 * Arg:            width [UNKN ] Undocumented argument [int]
 * Arg:             tail [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
# line 209 "hsp.dy"
HSPset * new_consistent_HSPset(HSPset * set,int min_score,int width,int tail)
{
  HSPset * out;
  int i,j;
  int diag_a;
  int diag_b;
  int eaten;

  /*  fprintf(stderr,"Entering consistency for %s with %d\n",set->hsp[0]->target->name,set->len); */
  out = HSPset_alloc_std();

  sort_HSPset_by_score(set);


  add_HSPset(out,hard_link_HSP(set->hsp[0]));
  out->score = set->hsp[0]->score;
  
  for(i=1;i<set->len;i++) {
    /* check against exisiting HSPs. If fits, add into big set */
    eaten = 0;

    for(j=0;j<out->len;j++) {
      diag_a = set->hsp[i]->query_start - set->hsp[i]->target_start;
      diag_b = out->hsp[j]->query_start - out->hsp[j]->target_start;
      if( abs(diag_a - diag_b) > 2 * width ) {
	continue; /* does not match */
      }

      eaten = 1;
      add_HSPset(out,hard_link_HSP(set->hsp[i]));
      out->score += set->hsp[i]->score;
      break;
    }
  }

  

  return out;

}

/* Function:  new_LinearHSPmanager_heuristic_max(hspm,max_size)
 *
 * Descrip:    Builds a new LinearHSPmanager from hash based with heuristics 
 *
 *
 * Arg:            hspm [UNKN ] Undocumented argument [HSPmanager *]
 * Arg:        max_size [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
# line 253 "hsp.dy"
LinearHSPmanager * new_LinearHSPmanager_heuristic_max(HSPmanager * hspm,int max_size)
{
  LinearHSPmanager * temp;
  LinearHSPmanager * out;

  assert(hspm);
  assert(max_size > 0);

  temp = new_LinearHSPmanager_truncate_on_score(hspm);

  out = truncated_LinearHSPmanager(temp,max_size,80,30,40);

  /*  free_LinearHSPmanager(temp); */

  return out;
}

/* Function:  new_LinearHSPmanager_truncate_on_score(hspm)
 *
 * Descrip:    Builds a LinearHSPmanager from a hash based HSP manager, using worst score truncation
 *
 *
 * Arg:        hspm [UNKN ] Undocumented argument [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
# line 273 "hsp.dy"
LinearHSPmanager * new_LinearHSPmanager_truncate_on_score(HSPmanager * hspm)
{
  LinearHSPmanager * out;


  out = LinearHSPmanager_alloc_std();
  out->mat = hard_link_CompMat(hspm->mat);

  out->worst_hsp_score = hspm->tsm->worst_score;
/*
  for(i=0;i<hspm->tsm->length;i++) {
    fprintf(stderr,"At position %d got score %d\n",i,hspm->tsm->score[i]);
  }
*/
  /*
  fprintf(stderr,"Before management, we have %d, worst score is %d\n",g_hash_table_size(hspm->target_hash),out->worst_hsp_score);
  */
  g_hash_table_foreach(hspm->target_hash,linearise_HSPset_truncate_on_score,out);

/*  sort_LinearHSPmanager(out,compare_HSPset_score); */
    
  return out;

}


/* Function:  new_LinearHSPmanager_flat(hspm)
 *
 * Descrip:    Builds a LinearHSPmanager from a hash based HSP manager
 *
 *
 * Arg:        hspm [UNKN ] Undocumented argument [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
# line 302 "hsp.dy"
LinearHSPmanager * new_LinearHSPmanager_flat(HSPmanager * hspm)
{
  LinearHSPmanager * out;

  out = LinearHSPmanager_alloc_std();
  out->mat = hard_link_CompMat(hspm->mat);


  g_hash_table_foreach(hspm->target_hash,linearise_HSPset_flat,out);

  sort_LinearHSPmanager(out,compare_HSPset_score);
    
  return out;

}

/* Function:  linearise_HSPset_truncate_on_score(key,value,user_data)
 *
 * Descrip:    internal function for remapping HSPs
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
# line 321 "hsp.dy"
void linearise_HSPset_truncate_on_score(gpointer key,gpointer value,gpointer user_data)
{
  LinearHSPmanager * l = (LinearHSPmanager *) user_data;
  HSPset * s = (HSPset *) value;

  if( s->score < l->worst_hsp_score ) {
    return;
  } else {
    add_LinearHSPmanager(l,hard_link_HSPset(s));
  }
}


/* Function:  linearise_HSPset_flat(key,value,user_data)
 *
 * Descrip:    internal function for remapping HSPs with score cutoff
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
# line 337 "hsp.dy"
void linearise_HSPset_flat(gpointer key,gpointer value,gpointer user_data)
{
  LinearHSPmanager * l = (LinearHSPmanager *) user_data;
  HSPset * s = (HSPset *) value;
  int i;
  
  s->score = 0;
  s->best_score =0;


  for(i=0;i<s->len;i++) {
    s->score += s->hsp[i]->score;
    if( s->hsp[i]->score > s->best_score ) {
      s->best_score = s->hsp[i]->score;
    }
  }

  add_LinearHSPmanager(l,hard_link_HSPset(s));

}

/* Function:  linearise_HSPset_consistent(key,value,user_data)
 *
 * Descrip:    internal function for remapping HSPs
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
# line 361 "hsp.dy"
void linearise_HSPset_consistent(gpointer key,gpointer value,gpointer user_data)
{
  LinearHSPmanager * l = (LinearHSPmanager *) user_data;
  HSPset * s = (HSPset *) value;
  HSPset * add;

  add = new_consistent_HSPset(s,l->min_score,l->width,l->tail);
  if( add->len > 0 ) {
    add_LinearHSPmanager(l,add);
  } else {
    free_HSPset(add);
  }
}

/* Function:  compare_HSPset_score(one,two)
 *
 * Descrip:    internal function for sort linear HSPsets
 *
 *
 * Arg:        one [UNKN ] Undocumented argument [HSPset *]
 * Arg:        two [UNKN ] Undocumented argument [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
# line 378 "hsp.dy"
int compare_HSPset_score(HSPset * one,HSPset * two)
{
  return two->score - one->score;
}



/* Function:  new_HSPmanager(query,mat,drop_off)
 *
 * Descrip:    Builds a new HSPmanager for a target system
 *
 *
 * Arg:           query [UNKN ] Undocumented argument [Sequence *]
 * Arg:             mat [UNKN ] Undocumented argument [CompMat *]
 * Arg:        drop_off [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
# line 388 "hsp.dy"
HSPmanager * new_HSPmanager(Sequence * query,CompMat * mat,int drop_off)
{
  HSPmanager * out;

  out = HSPmanager_alloc();

  out->query = hard_link_Sequence(query);
  if( mat == NULL ) {
    out->mat = NULL;
  } else {
    out->mat   = hard_link_CompMat(mat);
  }
  /*  out->qs = new_QuerySeqHSP(query,mat); */
  out->qs = NULL;

  out->drop_off = drop_off;
  out->target_hash =  g_hash_table_new(g_direct_hash,g_direct_equal);
  out->tsm = new_TopScoreManager(250);
  out->cache = new_HSPCache(2000);
  out->min_score = 25;
  
  return out;
}

/* Function:  sort_HSPset_by_score(*set)
 *
 * Descrip:    Sorts by score
 *
 *
 * Arg:        *set [UNKN ] Undocumented argument [HSPset]
 *
 */
# line 415 "hsp.dy"
void sort_HSPset_by_score(HSPset *set)
{
  sort_HSPset(set,compare_HSP_score);
}

/* Function:  compare_HSP_score(one,two)
 *
 * Descrip:    internal for sort by score
 *
 *
 * Arg:        one [UNKN ] Undocumented argument [HSP *]
 * Arg:        two [UNKN ] Undocumented argument [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
# line 424 "hsp.dy"
int compare_HSP_score(HSP * one,HSP * two)
{
  return two->score - one->score;
}


/* Function:  add_pair_HSPmanager(hspm,target,query_pos,target_pos)
 *
 * Descrip:    Adds a new target pair to this HSPmanager for indexing
 *
 *
 * Arg:              hspm [UNKN ] Undocumented argument [HSPmanager *]
 * Arg:            target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
# line 433 "hsp.dy"
boolean add_pair_HSPmanager(HSPmanager * hspm,Sequence * target,int query_pos,int target_pos)
{
  HSPset * set;
  int i;
  HSP * hsp;
  boolean has_set = 0;

  /* see if this target is loaded into the manager */

  if( (set = g_hash_table_lookup(hspm->target_hash,(gpointer)target)) != NULL ) {
    has_set = 1;
  }

  /* set is now the HSPset. Ensure this position is not already accounted for
     in the set */

  if( has_set == 1 ) {
    if( set->last_accessed != -1 && on_HSP(set->hsp[set->last_accessed],query_pos,target_pos) ) {
      return TRUE;
    }

    for(i=0;i<set->len;i++) {
      if( on_HSP(set->hsp[i],query_pos,target_pos) == TRUE ) {
	set->last_accessed = i;
	return TRUE;
      } 
    }
  } 

  if( hspm->qs != NULL ) {
    hsp = new_HSP_QuerySeqHSP(hspm->cache,hspm->qs,target,query_pos,target_pos,hspm->mat,hspm->drop_off);
  } else {
    hsp = new_HSP(hspm->cache,hspm->query,target,query_pos,target_pos,hspm->mat,hspm->drop_off);
  }

  if( hsp->score < hspm->min_score ) {
    free_HSP(hsp);
    return TRUE;
  }


  if( has_set == 0 )  {
    set = HSPset_alloc_std();
    g_hash_table_insert(hspm->target_hash,(gpointer)target,set);
  }


  set->score += hsp->score;
  if( hsp->score > set->best_score ) {
    set->best_score = hsp->score;
    add_score_TopScoreManager(hspm->tsm,set->best_score);
  }

 
  add_HSPset(set,hsp);
  set->last_accessed = set->len-1;

  return TRUE;
}

/* Function:  add_new_HSP_HSPmanager(hspm,target,query_start,target_start,length,score)
 *
 * Descrip:    Adds a new HSP when all info is known
 *
 *
 * Arg:                hspm [UNKN ] Undocumented argument [HSPmanager *]
 * Arg:              target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_start [UNKN ] Undocumented argument [int]
 * Arg:        target_start [UNKN ] Undocumented argument [int]
 * Arg:              length [UNKN ] Undocumented argument [int]
 * Arg:               score [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
# line 496 "hsp.dy"
boolean add_new_HSP_HSPmanager(HSPmanager * hspm,Sequence * target,int query_start,int target_start,int length,int score)
{
  HSPset * set;
  HSP * hsp;

  /* see if this target is loaded into the manager */

  if( (set = g_hash_table_lookup(hspm->target_hash,(gpointer)target)) == NULL ) {
    set = HSPset_alloc_std();
    g_hash_table_insert(hspm->target_hash,(gpointer)target,set);
  }


  if( hspm->cache != NULL ) {
    hsp = HSP_alloc_cache(hspm->cache);
  } else {
    hsp = HSP_alloc();
  }

  hsp->query = hard_link_Sequence(hspm->query);
  hsp->target= hard_link_Sequence(target);
  hsp->score = score;
  hsp->target_start = target_start;
  hsp->query_start = query_start;
  hsp->length = length;


  add_HSPset(set,hsp);

  return TRUE;
}

/* Function:  free_ghash_HSPsets(key,value,user_data)
 *
 * Descrip:    Frees the HSPsets
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
# line 531 "hsp.dy"
void free_ghash_HSPsets(gpointer key,gpointer value,gpointer user_data)
{
  int i;
  HSPset * val = (HSPset *) value;
  HSPCache * cache = (HSPCache*) user_data;

  if( val->dynamite_hard_link > 1 ) {
    val->dynamite_hard_link--;
    return;
  }

  if( cache != NULL ) {
    for(i=0;i<val->len;i++) {
      free_HSP_cache(cache,val->hsp[i]);
    }
    val->len = 0;
  }

  free_HSPset(val);
}

/* Function:  free_HSPmanager(h)
 *
 * Descrip:    Frees the HSPmanager
 *
 *
 * Arg:        h [UNKN ] Undocumented argument [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
# line 556 "hsp.dy"
HSPmanager * free_HSPmanager(HSPmanager * h)
{
  g_hash_table_foreach(h->target_hash,free_ghash_HSPsets,h->cache);
  g_hash_table_destroy(h->target_hash);
  free_CompMat(h->mat);
  if( h->tsm != NULL ) {
    free_TopScoreManager(h->tsm);
  }
  if( h->qs != NULL ) {
    free_QuerySeqHSP(h->qs);
  }

  ckfree(h);
  return NULL;
}



/* Function:  on_HSP(test,query_pos,target_pos)
 *
 * Descrip:    tests whether this point is on this test HSP
 *
 *
 * Arg:              test [UNKN ] Undocumented argument [HSP *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
# line 577 "hsp.dy"
boolean on_HSP(HSP * test,int query_pos,int target_pos)
{
  if( (test->query_start - test->target_start) != query_pos - target_pos || query_pos < test->query_start || target_pos < test->target_start) {
    return FALSE;
  }

  if( query_pos - test->query_start > test->length ) {
    return FALSE;
  }
  return TRUE;
}


/* Function:  new_HSP_QuerySeqHSP(cache,query,target,query_pos,target_pos,mat,drop_off)
 *
 * Descrip:    builds a new HSP for these sequences
 *
 *
 * Arg:             cache [UNKN ] Undocumented argument [HSPCache *]
 * Arg:             query [UNKN ] Undocumented argument [QuerySeqHSP *]
 * Arg:            target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 * Arg:               mat [UNKN ] Undocumented argument [CompMat *]
 * Arg:          drop_off [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
# line 593 "hsp.dy"
HSP * new_HSP_QuerySeqHSP(HSPCache * cache,QuerySeqHSP * query,Sequence * target,int query_pos,int target_pos,CompMat * mat,int drop_off)
{
  int i,j;
  int ii,jj;
  int temp_score;
  int t;
  int pause_i,pause_j;
  int score = 0;
  HSP * out;

  if( cache != NULL ) {
    out = HSP_alloc_cache(cache);
  } else {
    out = HSP_alloc();
  }

  out->query = hard_link_Sequence(query->query);
  out->target= hard_link_Sequence(target);
  out->score = -(query->score[query_pos][target->seq[target_pos]-'A']);


  pause_i = i = query_pos;
  pause_j = j = target_pos;

  /* go upstream first */
  
  for(score=0;i >= 0 && j >= 0;i--,j--) {

    t= query->score[i][target->seq[j]-'A'];

    if( t >= 0 ) {
      /* this is a positive score, we are on a high scoring run, so add and move on*/
      score += t;
    } else {
      /* negative score. i+1,j+1 was our last best score */
      for(temp_score = t,ii=i-1,jj=j-1;temp_score >= -drop_off && ii >= 0 && jj >= 0 && temp_score < 0;ii--,jj--) {

	temp_score += query->score[ii][target->seq[jj]-'A'];
      }
      
      if( temp_score >= 0 ) {
	/* new maximum reached */
	i = ii;
	j = jj;
	score += temp_score;
	continue; /* back to main loop */
      } else {
	/* either temp_score < -drop_off or something else */
	break;
      }
    }
  }
  
  /* set start position */
  out->query_start = i+1;
  out->target_start = j+1;
  out->score += score;

  /* downstream */


  for(score=0,i=query_pos,j=target_pos;i < query->len && j < target->len;i++,j++) {

    t= query->score[i][target->seq[j]-'A'];
    /*    fprintf(stderr,"Doing %d %d, [%c,%c] off %d score %d\n",i,j,query->query->seq[i],target->seq[j],t,score); */
    if( t >= 0 ) {
      /* this is a positive score, we are on a high scoring run, so add and move on*/
      score += t;
    } else {
      /* negative score. i-1,j-1 was our last best score */
      for(temp_score = t,ii=i+1,jj=j+1;temp_score >= -drop_off && ii < query->len && jj < target->len && temp_score < 0;ii++,jj++) {
	temp_score += query->score[ii][target->seq[jj]-'A'];
	/*	fprintf(stderr,"Negative looping %d,%d temp %d [%c,%c]\n",ii,jj,temp_score,query->query->seq[ii],target->seq[jj]); */
	
      }
      
      if( temp_score >= 0 ) {
	/* new good score reached */
	i = ii;
	j = jj;
	score += temp_score;
	continue; /* back to main loop */
      } else {
	/* either temp_score < -drop_off or something else */
	break;
      }
    }
  }
      
  /* set length and score */
  out->length = i-1 - out->query_start +1; 
  out->score += score;

	  
  /*
   * Old Style
  for(score = 0,max_score = 0;(max_score - score) < drop_off && i >= 0 && j >= 0;i--,j--) {
    score += query->score[i][target->seq[j]-'A'];
    if( score > max_score ) {
      pause_i = i;
      pause_j = j;
      max_score = score;
    }
  }
  

  out->query_start = pause_i;
  out->target_start = pause_j;
  out->score += max_score;

  */

  /* now downstream */

  /*
   * Old style

  for(score=0,max_score=0,pause_i=i=query_pos,pause_j=j=target_pos;(max_score - score) < drop_off && i < query->len && j < target->len;i++,j++) {
    score += query->score[i][target->seq[j]-'A'];
    if( score > max_score ) {
      pause_i = i;
      pause_j = j;
      max_score = score;
    }
  }

  out->length = pause_i - out->query_start+1;

  out->score += max_score;

  */





  return out;
}


/* Function:  new_HSP(cache,query,target,query_pos,target_pos,mat,drop_off)
 *
 * Descrip:    builds a new HSP for these sequences
 *
 *
 * Arg:             cache [UNKN ] Undocumented argument [HSPCache *]
 * Arg:             query [UNKN ] Undocumented argument [Sequence *]
 * Arg:            target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 * Arg:               mat [UNKN ] Undocumented argument [CompMat *]
 * Arg:          drop_off [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
# line 736 "hsp.dy"
HSP * new_HSP(HSPCache * cache,Sequence * query,Sequence * target,int query_pos,int target_pos,CompMat * mat,int drop_off)
{
  int i,j;
  int pause_i,pause_j;
  int score = 0;
  int max_score= 0;
  HSP * out;



  if( cache != NULL ) {
    out = HSP_alloc_cache(cache);
  } else {
    out = HSP_alloc();
  }

  assert(out);
  assert(query);
  assert(query->seq);
  assert(target);
  assert(target->seq);
  assert(mat);
  assert(drop_off > 0);

  /*  fprintf(stdout,"got new hsp with drop_off of %d\n",drop_off);*/

  out->query = hard_link_Sequence(query);
  out->target= hard_link_Sequence(target);
  out->score = -(mat->comp[query->seq[query_pos]-'A'][target->seq[target_pos]-'A']);

  pause_i = i = query_pos;
  pause_j = j = target_pos;

  /* go upstream first */
  
  for(score = 0,max_score = 0;(max_score - score) < drop_off && i >= 0 && j >= 0;i--,j--) {

    score += mat->comp[query->seq[i]-'A'][target->seq[j]-'A'];
    /*    fprintf(stdout,"Examining %c,%c - score %d max_score %d\n",query->seq[i],target->seq[j],score,max_score); */

    if( score > max_score ) {
      pause_i = i;
      pause_j = j;
      max_score = score;
    }
  }
  

  out->query_start = pause_i;
  out->target_start = pause_j;
  out->score += max_score;

  /* now downstream */

  for(score=0,max_score=0,pause_i=i=query_pos,pause_j=j=target_pos;(max_score - score) < drop_off && i < query->len && j < target->len;i++,j++) {
    score += mat->comp[query->seq[i]-'A'][target->seq[j]-'A'];
    if( score > max_score ) {
      pause_i = i;
      pause_j = j;
      max_score = score;
    }
  }

  out->length = pause_i - out->query_start+1;

  out->score += max_score;

  return out;
}

/* Function:  HSP_alloc_cache(hspc)
 *
 * Descrip:    Returns new HSP, using cache if needed
 *
 *
 * Arg:        hspc [UNKN ] Undocumented argument [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
# line 809 "hsp.dy"
HSP * HSP_alloc_cache(HSPCache * hspc)
{
  assert(hspc);

  if( hspc->len > 0 ) {
    hspc->len--;
    return hspc->cache[hspc->len];
  } 

  return HSP_alloc();
}

/* Function:  free_HSP_cache(cache,hsp)
 *
 * Descrip:    Places HSP back into cache, freeing if necessary
 *
 *
 * Arg:        cache [UNKN ] Undocumented argument [HSPCache *]
 * Arg:          hsp [UNKN ] Undocumented argument [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
# line 824 "hsp.dy"
HSP * free_HSP_cache(HSPCache * cache,HSP * hsp)
{
  int return_early = 0;

  assert(cache);
  assert(hsp);

#ifdef PTHREAD   
  assert(pthread_mutex_lock(&(hsp->dynamite_mutex)) == 0); 
#endif   
  if( hsp->dynamite_hard_link > 1)     {  
    return_early = 1;  
    hsp->dynamite_hard_link--; 
  }  
#ifdef PTHREAD   
  assert(pthread_mutex_unlock(&(hsp->dynamite_mutex)) == 0);   
#endif   
  
  if( return_early == 1)   
    return NULL;   
  
  if( cache->len > cache->max_cache ) {
    return free_HSP(hsp);
  } else {
    add_HSPCache(cache,hsp);
    return NULL;
  }
}

/* Function:  new_HSPCache(maxsize)
 *
 * Descrip:    Makes a new cache
 *
 *
 * Arg:        maxsize [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
# line 856 "hsp.dy"
HSPCache * new_HSPCache(int maxsize)
{
  HSPCache * out;

  out = HSPCache_alloc_std();
  out->max_cache = maxsize;

  return out;
}

/* Function:  show_HSPset(s,ofp)
 *
 * Descrip:    Shows a HSP set
 *
 *
 * Arg:          s [UNKN ] Undocumented argument [HSPset *]
 * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
 *
 */
# line 869 "hsp.dy"
void show_HSPset(HSPset * s,FILE * ofp)
{
  int i;

  for(i=0;i<s->len;i++) {
    fprintf(ofp,"%d %d %d\n",s->hsp[i]->query_start,s->hsp[i]->target_start,s->hsp[i]->length);
  }


}

/* Function:  show_HSP(hsp,linelength,out)
 *
 * Descrip:    Shows an HSP
 *
 *
 * Arg:               hsp [UNKN ] Undocumented argument [HSP *]
 * Arg:        linelength [UNKN ] Undocumented argument [int]
 * Arg:               out [UNKN ] Undocumented argument [FILE *]
 *
 */
# line 883 "hsp.dy"
void show_HSP(HSP * hsp,int linelength,FILE * out)
{
  int i,j;

  for(i=hsp->query_start,j=hsp->target_start;(i-hsp->query_start) < hsp->length;i++,j++) {
    fprintf(out,"%c %c\n",hsp->query->seq[i],hsp->target->seq[j]);
  }

}


/* Function:  new_QuerySeqHSP(seq,mat)
 *
 * Descrip:    Builds a new QuerySeqHSP from Sequence and Matrix
 *
 *
 * Arg:        seq [UNKN ] Undocumented argument [Sequence *]
 * Arg:        mat [UNKN ] Undocumented argument [CompMat *]
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
# line 897 "hsp.dy"
QuerySeqHSP * new_QuerySeqHSP(Sequence * seq,CompMat * mat)
{
  QuerySeqHSP * out;
  int i;

  assert(seq);
  assert(mat);

  out = QuerySeqHSP_alloc();
  out->query = hard_link_Sequence(seq);
  out->score = (int **)calloc(seq->len,sizeof(int*));
  out->len = seq->len;
  
  for(i=0;i<seq->len;i++) {
    out->score[i] = mat->comp[toupper(seq->seq[i])-'A'];
  }

  return out;
}
# line 1039 "hsp.c"
/* Function:  hard_link_HSP(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * hard_link_HSP(HSP * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a HSP object: passed a NULL object"); 
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  HSP_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * HSP_alloc(void) 
{
    HSP * out;  /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(HSP *) ckalloc (sizeof(HSP))) == NULL)  {  
      warn("HSP_alloc failed "); 
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->query = NULL;   
    out->target = NULL;  
    out->query_start = 0;    
    out->target_start = 0;   
    out->length = 0; 
    out->score = 0;  
    out->is_in_block = 0;    


    return out;  
}    


/* Function:  free_HSP(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * free_HSP(HSP * obj) 
{
    int return_early = 0;    


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a HSP obj. Should be trappable");   
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->query != NULL)  
      free_Sequence(obj->query);     
    if( obj->target != NULL) 
      free_Sequence(obj->target);    


    ckfree(obj); 
    return NULL; 
}    


/* Function:  swap_HSPCache(list,i,j)
 *
 * Descrip:    swap function: an internal for qsort_HSPCache
 *             swaps two positions in the array
 *
 *
 * Arg:        list [UNKN ] List of structures to swap in [HSP **]
 * Arg:           i [UNKN ] swap position [int]
 * Arg:           j [UNKN ] swap position [int]
 *
 */
/* swap function for qsort function */ 
void swap_HSPCache(HSP ** list,int i,int j)  
{
    HSP * temp;  
    temp=list[i];    
    list[i]=list[j]; 
    list[j]=temp;    
}    


/* Function:  qsort_HSPCache(list,left,right,comp)
 *
 * Descrip:    qsort - lifted from K&R 
 *             sorts the array using quicksort
 *             Probably much better to call sort_HSPCache which sorts from start to end
 *
 *
 * Arg:         list [UNKN ] List of structures to swap in [HSP **]
 * Arg:         left [UNKN ] left position [int]
 * Arg:        right [UNKN ] right position [int]
 * Arg:         comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void qsort_HSPCache(HSP ** list,int left,int right,int (*comp)(HSP * ,HSP * )) 
{
    int i,last;  
    if( left >= right )  
      return;    


    swap_HSPCache(list,left,(left+right)/2); 
    last = left; 
    for ( i=left+1; i <= right;i++)  {  
      if( (*comp)(list[i],list[left]) < 0)   
        swap_HSPCache (list,++last,i);   
      }  
    swap_HSPCache (list,left,last);  
    qsort_HSPCache(list,left,last-1,comp);   
    qsort_HSPCache(list,last+1,right,comp);  
}    


/* Function:  sort_HSPCache(obj,comp)
 *
 * Descrip:    sorts from start to end using comp 
 *             sorts the array using quicksort by calling qsort_HSPCache
 *
 *
 * Arg:         obj [UNKN ] Object containing list [HSPCache *]
 * Arg:        comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void sort_HSPCache(HSPCache * obj,int (*comp)(HSP *, HSP *)) 
{
    qsort_HSPCache(obj->cache,0,obj->len-1,comp);    
    return;  
}    


/* Function:  expand_HSPCache(obj,len)
 *
 * Descrip:    Really an internal function for add_HSPCache
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [HSPCache *]
 * Arg:        len [UNKN ] Length to add one [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean expand_HSPCache(HSPCache * obj,int len) 
{


    if( obj->maxlen > obj->len )     {  
      warn("expand_HSPCache called with no need");   
      return TRUE;   
      }  


    if( (obj->cache = (HSP ** ) ckrealloc (obj->cache,sizeof(HSP *)*len)) == NULL)   {  
      warn("ckrealloc failed for expand_HSPCache, returning FALSE"); 
      return FALSE;  
      }  
    obj->maxlen = len;   
    return TRUE; 
}    


/* Function:  add_HSPCache(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [HSPCache *]
 * Arg:        add [OWNER] Object to add to the list [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
/* will expand function if necessary */ 
boolean add_HSPCache(HSPCache * obj,HSP * add) 
{
    if( obj->len >= obj->maxlen) {  
      if( expand_HSPCache(obj,obj->len + HSPCacheLISTLENGTH) == FALSE)   
        return FALSE;    
      }  


    obj->cache[obj->len++]=add;  
    return TRUE; 
}    


/* Function:  flush_HSPCache(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int flush_HSPCache(HSPCache * obj) 
{
    int i;   


    for(i=0;i<obj->len;i++)  { /*for i over list length*/ 
      if( obj->cache[i] != NULL) {  
        free_HSP(obj->cache[i]); 
        obj->cache[i] = NULL;    
        }  
      } /* end of for i over list length */ 


    obj->len = 0;    
    return i;    
}    


/* Function:  HSPCache_alloc_std(void)
 *
 * Descrip:    Equivalent to HSPCache_alloc_len(HSPCacheLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * HSPCache_alloc_std(void) 
{
    return HSPCache_alloc_len(HSPCacheLISTLENGTH);   
}    


/* Function:  HSPCache_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * HSPCache_alloc_len(int len) 
{
    HSPCache * out; /* out is exported at the end of function */ 


    /* Call alloc function: return NULL if NULL */ 
    /* Warning message alread in alloc function */ 
    if((out = HSPCache_alloc()) == NULL) 
      return NULL;   


    /* Calling ckcalloc for list elements */ 
    if((out->cache = (HSP ** ) ckcalloc (len,sizeof(HSP *))) == NULL)    {  
      warn("Warning, ckcalloc failed in HSPCache_alloc_len");    
      return NULL;   
      }  
    out->len = 0;    
    out->maxlen = len;   


    return out;  
}    


/* Function:  hard_link_HSPCache(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * hard_link_HSPCache(HSPCache * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a HSPCache object: passed a NULL object");    
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  HSPCache_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * HSPCache_alloc(void) 
{
    HSPCache * out; /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(HSPCache *) ckalloc (sizeof(HSPCache))) == NULL)    {  
      warn("HSPCache_alloc failed ");    
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->cache = NULL;   
    out->len = out->maxlen = 0;  
    out->max_cache = 0;  


    return out;  
}    


/* Function:  free_HSPCache(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * free_HSPCache(HSPCache * obj) 
{
    int return_early = 0;    
    int i;   


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a HSPCache obj. Should be trappable");  
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->cache != NULL)  {  
      for(i=0;i<obj->len;i++)    {  
        if( obj->cache[i] != NULL)   
          free_HSP(obj->cache[i]);   
        }  
      ckfree(obj->cache);    
      }  


    ckfree(obj); 
    return NULL; 
}    


/* Function:  swap_HSPset(list,i,j)
 *
 * Descrip:    swap function: an internal for qsort_HSPset
 *             swaps two positions in the array
 *
 *
 * Arg:        list [UNKN ] List of structures to swap in [HSP **]
 * Arg:           i [UNKN ] swap position [int]
 * Arg:           j [UNKN ] swap position [int]
 *
 */
/* swap function for qsort function */ 
void swap_HSPset(HSP ** list,int i,int j)  
{
    HSP * temp;  
    temp=list[i];    
    list[i]=list[j]; 
    list[j]=temp;    
}    


/* Function:  qsort_HSPset(list,left,right,comp)
 *
 * Descrip:    qsort - lifted from K&R 
 *             sorts the array using quicksort
 *             Probably much better to call sort_HSPset which sorts from start to end
 *
 *
 * Arg:         list [UNKN ] List of structures to swap in [HSP **]
 * Arg:         left [UNKN ] left position [int]
 * Arg:        right [UNKN ] right position [int]
 * Arg:         comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void qsort_HSPset(HSP ** list,int left,int right,int (*comp)(HSP * ,HSP * )) 
{
    int i,last;  
    if( left >= right )  
      return;    


    swap_HSPset(list,left,(left+right)/2);   
    last = left; 
    for ( i=left+1; i <= right;i++)  {  
      if( (*comp)(list[i],list[left]) < 0)   
        swap_HSPset (list,++last,i); 
      }  
    swap_HSPset (list,left,last);    
    qsort_HSPset(list,left,last-1,comp); 
    qsort_HSPset(list,last+1,right,comp);    
}    


/* Function:  sort_HSPset(obj,comp)
 *
 * Descrip:    sorts from start to end using comp 
 *             sorts the array using quicksort by calling qsort_HSPset
 *
 *
 * Arg:         obj [UNKN ] Object containing list [HSPset *]
 * Arg:        comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void sort_HSPset(HSPset * obj,int (*comp)(HSP *, HSP *)) 
{
    qsort_HSPset(obj->hsp,0,obj->len-1,comp);    
    return;  
}    


/* Function:  expand_HSPset(obj,len)
 *
 * Descrip:    Really an internal function for add_HSPset
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [HSPset *]
 * Arg:        len [UNKN ] Length to add one [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean expand_HSPset(HSPset * obj,int len) 
{


    if( obj->maxlen > obj->len )     {  
      warn("expand_HSPset called with no need"); 
      return TRUE;   
      }  


    if( (obj->hsp = (HSP ** ) ckrealloc (obj->hsp,sizeof(HSP *)*len)) == NULL)   {  
      warn("ckrealloc failed for expand_HSPset, returning FALSE");   
      return FALSE;  
      }  
    obj->maxlen = len;   
    return TRUE; 
}    


/* Function:  add_HSPset(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [HSPset *]
 * Arg:        add [OWNER] Object to add to the list [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
/* will expand function if necessary */ 
boolean add_HSPset(HSPset * obj,HSP * add) 
{
    if( obj->len >= obj->maxlen) {  
      if( expand_HSPset(obj,obj->len + HSPsetLISTLENGTH) == FALSE)   
        return FALSE;    
      }  


    obj->hsp[obj->len++]=add;    
    return TRUE; 
}    


/* Function:  flush_HSPset(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int flush_HSPset(HSPset * obj) 
{
    int i;   


    for(i=0;i<obj->len;i++)  { /*for i over list length*/ 
      if( obj->hsp[i] != NULL)   {  
        free_HSP(obj->hsp[i]);   
        obj->hsp[i] = NULL;  
        }  
      } /* end of for i over list length */ 


    obj->len = 0;    
    return i;    
}    


/* Function:  HSPset_alloc_std(void)
 *
 * Descrip:    Equivalent to HSPset_alloc_len(HSPsetLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * HSPset_alloc_std(void) 
{
    return HSPset_alloc_len(HSPsetLISTLENGTH);   
}    


/* Function:  HSPset_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * HSPset_alloc_len(int len) 
{
    HSPset * out;   /* out is exported at the end of function */ 


    /* Call alloc function: return NULL if NULL */ 
    /* Warning message alread in alloc function */ 
    if((out = HSPset_alloc()) == NULL)   
      return NULL;   


    /* Calling ckcalloc for list elements */ 
    if((out->hsp = (HSP ** ) ckcalloc (len,sizeof(HSP *))) == NULL)  {  
      warn("Warning, ckcalloc failed in HSPset_alloc_len");  
      return NULL;   
      }  
    out->len = 0;    
    out->maxlen = len;   


    return out;  
}    


/* Function:  hard_link_HSPset(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * hard_link_HSPset(HSPset * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a HSPset object: passed a NULL object");  
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  HSPset_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * HSPset_alloc(void) 
{
    HSPset * out;   /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(HSPset *) ckalloc (sizeof(HSPset))) == NULL)    {  
      warn("HSPset_alloc failed ");  
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->hsp = NULL; 
    out->len = out->maxlen = 0;  
    out->score = 0;  
    out->best_score = 0; 
    out->last_accessed = -1; 


    return out;  
}    


/* Function:  free_HSPset(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * free_HSPset(HSPset * obj) 
{
    int return_early = 0;    
    int i;   


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a HSPset obj. Should be trappable");    
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->hsp != NULL)    {  
      for(i=0;i<obj->len;i++)    {  
        if( obj->hsp[i] != NULL) 
          free_HSP(obj->hsp[i]); 
        }  
      ckfree(obj->hsp);  
      }  


    ckfree(obj); 
    return NULL; 
}    


/* Function:  hard_link_TopScoreManager(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TopScoreManager *]
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
TopScoreManager * hard_link_TopScoreManager(TopScoreManager * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a TopScoreManager object: passed a NULL object"); 
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  TopScoreManager_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
TopScoreManager * TopScoreManager_alloc(void) 
{
    TopScoreManager * out;  /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(TopScoreManager *) ckalloc (sizeof(TopScoreManager))) == NULL)  {  
      warn("TopScoreManager_alloc failed "); 
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->score = NULL;   
    out->length = 0; 
    out->current_pos = 0;    
    out->worst_score = 0;    
    out->worst_position = 0; 


    return out;  
}    


/* Function:  free_TopScoreManager(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TopScoreManager *]
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
TopScoreManager * free_TopScoreManager(TopScoreManager * obj) 
{
    int return_early = 0;    


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a TopScoreManager obj. Should be trappable");   
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->score != NULL)  
      ckfree(obj->score);    


    ckfree(obj); 
    return NULL; 
}    


/* Function:  hard_link_QuerySeqHSP(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [QuerySeqHSP *]
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
QuerySeqHSP * hard_link_QuerySeqHSP(QuerySeqHSP * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a QuerySeqHSP object: passed a NULL object"); 
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  QuerySeqHSP_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
QuerySeqHSP * QuerySeqHSP_alloc(void) 
{
    QuerySeqHSP * out;  /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(QuerySeqHSP *) ckalloc (sizeof(QuerySeqHSP))) == NULL)  {  
      warn("QuerySeqHSP_alloc failed "); 
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->query = NULL;   
    out->len = 0;    


    return out;  
}    


/* Function:  free_QuerySeqHSP(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [QuerySeqHSP *]
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
QuerySeqHSP * free_QuerySeqHSP(QuerySeqHSP * obj) 
{
    int return_early = 0;    


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a QuerySeqHSP obj. Should be trappable");   
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->query != NULL)  
      free_Sequence(obj->query);     
    /* obj->score is linked in */ 


    ckfree(obj); 
    return NULL; 
}    


/* Function:  hard_link_HSPmanager(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
HSPmanager * hard_link_HSPmanager(HSPmanager * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a HSPmanager object: passed a NULL object");  
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  HSPmanager_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
HSPmanager * HSPmanager_alloc(void) 
{
    HSPmanager * out;   /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(HSPmanager *) ckalloc (sizeof(HSPmanager))) == NULL)    {  
      warn("HSPmanager_alloc failed ");  
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->query = NULL;   
    out->mat = NULL; 
    out->drop_off = 0;   
    out->min_score = 0;  
    out->tsm = NULL; 
    out->qs = NULL;  
    out->cache = NULL;   


    return out;  
}    


/* Function:  swap_LinearHSPmanager(list,i,j)
 *
 * Descrip:    swap function: an internal for qsort_LinearHSPmanager
 *             swaps two positions in the array
 *
 *
 * Arg:        list [UNKN ] List of structures to swap in [HSPset **]
 * Arg:           i [UNKN ] swap position [int]
 * Arg:           j [UNKN ] swap position [int]
 *
 */
/* swap function for qsort function */ 
void swap_LinearHSPmanager(HSPset ** list,int i,int j)  
{
    HSPset * temp;   
    temp=list[i];    
    list[i]=list[j]; 
    list[j]=temp;    
}    


/* Function:  qsort_LinearHSPmanager(list,left,right,comp)
 *
 * Descrip:    qsort - lifted from K&R 
 *             sorts the array using quicksort
 *             Probably much better to call sort_LinearHSPmanager which sorts from start to end
 *
 *
 * Arg:         list [UNKN ] List of structures to swap in [HSPset **]
 * Arg:         left [UNKN ] left position [int]
 * Arg:        right [UNKN ] right position [int]
 * Arg:         comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void qsort_LinearHSPmanager(HSPset ** list,int left,int right,int (*comp)(HSPset * ,HSPset * )) 
{
    int i,last;  
    if( left >= right )  
      return;    


    swap_LinearHSPmanager(list,left,(left+right)/2); 
    last = left; 
    for ( i=left+1; i <= right;i++)  {  
      if( (*comp)(list[i],list[left]) < 0)   
        swap_LinearHSPmanager (list,++last,i);   
      }  
    swap_LinearHSPmanager (list,left,last);  
    qsort_LinearHSPmanager(list,left,last-1,comp);   
    qsort_LinearHSPmanager(list,last+1,right,comp);  
}    


/* Function:  sort_LinearHSPmanager(obj,comp)
 *
 * Descrip:    sorts from start to end using comp 
 *             sorts the array using quicksort by calling qsort_LinearHSPmanager
 *
 *
 * Arg:         obj [UNKN ] Object containing list [LinearHSPmanager *]
 * Arg:        comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void sort_LinearHSPmanager(LinearHSPmanager * obj,int (*comp)(HSPset *, HSPset *)) 
{
    qsort_LinearHSPmanager(obj->set,0,obj->len-1,comp);  
    return;  
}    


/* Function:  expand_LinearHSPmanager(obj,len)
 *
 * Descrip:    Really an internal function for add_LinearHSPmanager
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [LinearHSPmanager *]
 * Arg:        len [UNKN ] Length to add one [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean expand_LinearHSPmanager(LinearHSPmanager * obj,int len) 
{


    if( obj->maxlen > obj->len )     {  
      warn("expand_LinearHSPmanager called with no need");   
      return TRUE;   
      }  


    if( (obj->set = (HSPset ** ) ckrealloc (obj->set,sizeof(HSPset *)*len)) == NULL)     {  
      warn("ckrealloc failed for expand_LinearHSPmanager, returning FALSE"); 
      return FALSE;  
      }  
    obj->maxlen = len;   
    return TRUE; 
}    


/* Function:  add_LinearHSPmanager(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [LinearHSPmanager *]
 * Arg:        add [OWNER] Object to add to the list [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
/* will expand function if necessary */ 
boolean add_LinearHSPmanager(LinearHSPmanager * obj,HSPset * add) 
{
    if( obj->len >= obj->maxlen) {  
      if( expand_LinearHSPmanager(obj,obj->len + LinearHSPmanagerLISTLENGTH) == FALSE)   
        return FALSE;    
      }  


    obj->set[obj->len++]=add;    
    return TRUE; 
}    


/* Function:  flush_LinearHSPmanager(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [LinearHSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int flush_LinearHSPmanager(LinearHSPmanager * obj) 
{
    int i;   


    for(i=0;i<obj->len;i++)  { /*for i over list length*/ 
      if( obj->set[i] != NULL)   {  
        free_HSPset(obj->set[i]);    
        obj->set[i] = NULL;  
        }  
      } /* end of for i over list length */ 


    obj->len = 0;    
    return i;    
}    


/* Function:  LinearHSPmanager_alloc_std(void)
 *
 * Descrip:    Equivalent to LinearHSPmanager_alloc_len(LinearHSPmanagerLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * LinearHSPmanager_alloc_std(void) 
{
    return LinearHSPmanager_alloc_len(LinearHSPmanagerLISTLENGTH);   
}    


/* Function:  LinearHSPmanager_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * LinearHSPmanager_alloc_len(int len) 
{
    LinearHSPmanager * out; /* out is exported at the end of function */ 


    /* Call alloc function: return NULL if NULL */ 
    /* Warning message alread in alloc function */ 
    if((out = LinearHSPmanager_alloc()) == NULL) 
      return NULL;   


    /* Calling ckcalloc for list elements */ 
    if((out->set = (HSPset ** ) ckcalloc (len,sizeof(HSPset *))) == NULL)    {  
      warn("Warning, ckcalloc failed in LinearHSPmanager_alloc_len");    
      return NULL;   
      }  
    out->len = 0;    
    out->maxlen = len;   


    return out;  
}    


/* Function:  hard_link_LinearHSPmanager(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [LinearHSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * hard_link_LinearHSPmanager(LinearHSPmanager * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a LinearHSPmanager object: passed a NULL object");    
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  LinearHSPmanager_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * LinearHSPmanager_alloc(void) 
{
    LinearHSPmanager * out; /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(LinearHSPmanager *) ckalloc (sizeof(LinearHSPmanager))) == NULL)    {  
      warn("LinearHSPmanager_alloc failed ");    
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->set = NULL; 
    out->len = out->maxlen = 0;  
    out->query = NULL;   
    out->mat = NULL; 
    out->min_score = 0;  
    out->width = 0;  
    out->tail = 0;   
    out->worst_hsp_score = 0;    


    return out;  
}    


/* Function:  free_LinearHSPmanager(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [LinearHSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * free_LinearHSPmanager(LinearHSPmanager * obj) 
{
    int return_early = 0;    
    int i;   


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a LinearHSPmanager obj. Should be trappable");  
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->set != NULL)    {  
      for(i=0;i<obj->len;i++)    {  
        if( obj->set[i] != NULL) 
          free_HSPset(obj->set[i]);  
        }  
      ckfree(obj->set);  
      }  
    if( obj->query != NULL)  
      free_Sequence(obj->query);     
    if( obj->mat != NULL)    
      free_CompMat(obj->mat);    


    ckfree(obj); 
    return NULL; 
}    



#ifdef _cplusplus
}
#endif
