#ifndef DYNAMITEhspHEADERFILE
#define DYNAMITEhspHEADERFILE
#ifdef _cplusplus
extern "C" {
#endif

#include "sequence.h"
#include "compmat.h"
#include "glib.h"

#define HSPsetLISTLENGTH 20
#define LinearHSPmanagerLISTLENGTH 128

#define HSP_BLOCK_SIZE 1024

#define UNFEASIBLY_LARGE_SCORE 1000000

#define HSPCacheLISTLENGTH 1024


struct Wise2_HSP {  
    int dynamite_hard_link;  
#ifdef PTHREAD   
    pthread_mutex_t dynamite_mutex;  
#endif   
    Sequence * query;    
    Sequence * target;   
    int query_start;     
    int target_start;    
    int length;  
    int score;   
    char is_in_block;    
    } ;  
/* HSP defined */ 
#ifndef DYNAMITE_DEFINED_HSP
typedef struct Wise2_HSP Wise2_HSP;
#define HSP Wise2_HSP
#define DYNAMITE_DEFINED_HSP
#endif


struct Wise2_HSPCache {  
    int dynamite_hard_link;  
#ifdef PTHREAD   
    pthread_mutex_t dynamite_mutex;  
#endif   
    HSP ** cache;    
    int len;/* len for above cache  */ 
    int maxlen; /* maxlen for above cache */ 
    int max_cache;   
    } ;  
/* HSPCache defined */ 
#ifndef DYNAMITE_DEFINED_HSPCache
typedef struct Wise2_HSPCache Wise2_HSPCache;
#define HSPCache Wise2_HSPCache
#define DYNAMITE_DEFINED_HSPCache
#endif


struct Wise2_HSPset {  
    int dynamite_hard_link;  
#ifdef PTHREAD   
    pthread_mutex_t dynamite_mutex;  
#endif   
    HSP ** hsp;  
    int len;/* len for above hsp  */ 
    int maxlen; /* maxlen for above hsp */ 
    int score;   
    int best_score;  
    int last_accessed;   
    } ;  
/* HSPset defined */ 
#ifndef DYNAMITE_DEFINED_HSPset
typedef struct Wise2_HSPset Wise2_HSPset;
#define HSPset Wise2_HSPset
#define DYNAMITE_DEFINED_HSPset
#endif


struct Wise2_TopScoreManager {  
    int dynamite_hard_link;  
#ifdef PTHREAD   
    pthread_mutex_t dynamite_mutex;  
#endif   
    int * score;     
    int length;  
    int current_pos;     
    int worst_score;     
    int worst_position;  
    } ;  
/* TopScoreManager defined */ 
#ifndef DYNAMITE_DEFINED_TopScoreManager
typedef struct Wise2_TopScoreManager Wise2_TopScoreManager;
#define TopScoreManager Wise2_TopScoreManager
#define DYNAMITE_DEFINED_TopScoreManager
#endif


struct Wise2_QuerySeqHSP {  
    int dynamite_hard_link;  
#ifdef PTHREAD   
    pthread_mutex_t dynamite_mutex;  
#endif   
    Sequence * query;    
    int ** score;    
    int len;     
    } ;  
/* QuerySeqHSP defined */ 
#ifndef DYNAMITE_DEFINED_QuerySeqHSP
typedef struct Wise2_QuerySeqHSP Wise2_QuerySeqHSP;
#define QuerySeqHSP Wise2_QuerySeqHSP
#define DYNAMITE_DEFINED_QuerySeqHSP
#endif


struct Wise2_HSPmanager {  
    int dynamite_hard_link;  
#ifdef PTHREAD   
    pthread_mutex_t dynamite_mutex;  
#endif   
    GHashTable * target_hash;    
    Sequence * query;    
    CompMat * mat;   
    int drop_off;    
    int min_score;   
    TopScoreManager * tsm;   
    QuerySeqHSP * qs;    
    HSPCache * cache;    
    } ;  
/* HSPmanager defined */ 
#ifndef DYNAMITE_DEFINED_HSPmanager
typedef struct Wise2_HSPmanager Wise2_HSPmanager;
#define HSPmanager Wise2_HSPmanager
#define DYNAMITE_DEFINED_HSPmanager
#endif


struct Wise2_LinearHSPmanager {  
    int dynamite_hard_link;  
#ifdef PTHREAD   
    pthread_mutex_t dynamite_mutex;  
#endif   
    HSPset ** set;   
    int len;/* len for above set  */ 
    int maxlen; /* maxlen for above set */ 
    Sequence * query;    
    CompMat * mat;   
    int min_score;   
    int width;   
    int tail;    
    int worst_hsp_score;     
    } ;  
/* LinearHSPmanager defined */ 
#ifndef DYNAMITE_DEFINED_LinearHSPmanager
typedef struct Wise2_LinearHSPmanager Wise2_LinearHSPmanager;
#define LinearHSPmanager Wise2_LinearHSPmanager
#define DYNAMITE_DEFINED_LinearHSPmanager
#endif




    /***************************************************/
    /* Callable functions                              */
    /* These are the functions you are expected to use */
    /***************************************************/



/* Function:  new_TopScoreManager(length)
 *
 * Descrip:    Makes a new topscore manager of a specific length
 *
 *
 * Arg:        length [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
TopScoreManager * Wise2_new_TopScoreManager(int length);
#define new_TopScoreManager Wise2_new_TopScoreManager


/* Function:  add_score_TopScoreManager(tsm,score)
 *
 * Descrip:    Adds a top score
 *
 *
 * Arg:          tsm [UNKN ] Undocumented argument [TopScoreManager *]
 * Arg:        score [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean Wise2_add_score_TopScoreManager(TopScoreManager * tsm,int score);
#define add_score_TopScoreManager Wise2_add_score_TopScoreManager


/* Function:  truncated_LinearHSPmanager(lm,max_size,min_score,width,tail)
 *
 * Descrip:    Makes a truncated linear set 
 *
 *
 * Arg:               lm [UNKN ] Undocumented argument [LinearHSPmanager *]
 * Arg:         max_size [UNKN ] Undocumented argument [int]
 * Arg:        min_score [UNKN ] Undocumented argument [int]
 * Arg:            width [UNKN ] Undocumented argument [int]
 * Arg:             tail [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_truncated_LinearHSPmanager(LinearHSPmanager * lm,int max_size,int min_score,int width,int tail);
#define truncated_LinearHSPmanager Wise2_truncated_LinearHSPmanager


/* Function:  new_consistent_HSPset(set,min_score,width,tail)
 *
 * Descrip:    Makes an HSP set via heuristics
 *             to deal with low complexity regions
 *
 *
 * Arg:              set [UNKN ] Undocumented argument [HSPset *]
 * Arg:        min_score [UNKN ] Undocumented argument [int]
 * Arg:            width [UNKN ] Undocumented argument [int]
 * Arg:             tail [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * Wise2_new_consistent_HSPset(HSPset * set,int min_score,int width,int tail);
#define new_consistent_HSPset Wise2_new_consistent_HSPset


/* Function:  new_LinearHSPmanager_heuristic_max(hspm,max_size)
 *
 * Descrip:    Builds a new LinearHSPmanager from hash based with heuristics 
 *
 *
 * Arg:            hspm [UNKN ] Undocumented argument [HSPmanager *]
 * Arg:        max_size [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_new_LinearHSPmanager_heuristic_max(HSPmanager * hspm,int max_size);
#define new_LinearHSPmanager_heuristic_max Wise2_new_LinearHSPmanager_heuristic_max


/* Function:  new_LinearHSPmanager_truncate_on_score(hspm)
 *
 * Descrip:    Builds a LinearHSPmanager from a hash based HSP manager, using worst score truncation
 *
 *
 * Arg:        hspm [UNKN ] Undocumented argument [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_new_LinearHSPmanager_truncate_on_score(HSPmanager * hspm);
#define new_LinearHSPmanager_truncate_on_score Wise2_new_LinearHSPmanager_truncate_on_score


/* Function:  new_LinearHSPmanager_flat(hspm)
 *
 * Descrip:    Builds a LinearHSPmanager from a hash based HSP manager
 *
 *
 * Arg:        hspm [UNKN ] Undocumented argument [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_new_LinearHSPmanager_flat(HSPmanager * hspm);
#define new_LinearHSPmanager_flat Wise2_new_LinearHSPmanager_flat


/* Function:  linearise_HSPset_truncate_on_score(key,value,user_data)
 *
 * Descrip:    internal function for remapping HSPs
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
void Wise2_linearise_HSPset_truncate_on_score(gpointer key,gpointer value,gpointer user_data);
#define linearise_HSPset_truncate_on_score Wise2_linearise_HSPset_truncate_on_score


/* Function:  linearise_HSPset_flat(key,value,user_data)
 *
 * Descrip:    internal function for remapping HSPs with score cutoff
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
void Wise2_linearise_HSPset_flat(gpointer key,gpointer value,gpointer user_data);
#define linearise_HSPset_flat Wise2_linearise_HSPset_flat


/* Function:  linearise_HSPset_consistent(key,value,user_data)
 *
 * Descrip:    internal function for remapping HSPs
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
void Wise2_linearise_HSPset_consistent(gpointer key,gpointer value,gpointer user_data);
#define linearise_HSPset_consistent Wise2_linearise_HSPset_consistent


/* Function:  compare_HSPset_score(one,two)
 *
 * Descrip:    internal function for sort linear HSPsets
 *
 *
 * Arg:        one [UNKN ] Undocumented argument [HSPset *]
 * Arg:        two [UNKN ] Undocumented argument [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int Wise2_compare_HSPset_score(HSPset * one,HSPset * two);
#define compare_HSPset_score Wise2_compare_HSPset_score


/* Function:  new_HSPmanager(query,mat,drop_off)
 *
 * Descrip:    Builds a new HSPmanager for a target system
 *
 *
 * Arg:           query [UNKN ] Undocumented argument [Sequence *]
 * Arg:             mat [UNKN ] Undocumented argument [CompMat *]
 * Arg:        drop_off [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
HSPmanager * Wise2_new_HSPmanager(Sequence * query,CompMat * mat,int drop_off);
#define new_HSPmanager Wise2_new_HSPmanager


/* Function:  sort_HSPset_by_score(*set)
 *
 * Descrip:    Sorts by score
 *
 *
 * Arg:        *set [UNKN ] Undocumented argument [HSPset]
 *
 */
void Wise2_sort_HSPset_by_score(HSPset *set);
#define sort_HSPset_by_score Wise2_sort_HSPset_by_score


/* Function:  add_pair_HSPmanager(hspm,target,query_pos,target_pos)
 *
 * Descrip:    Adds a new target pair to this HSPmanager for indexing
 *
 *
 * Arg:              hspm [UNKN ] Undocumented argument [HSPmanager *]
 * Arg:            target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean Wise2_add_pair_HSPmanager(HSPmanager * hspm,Sequence * target,int query_pos,int target_pos);
#define add_pair_HSPmanager Wise2_add_pair_HSPmanager


/* Function:  add_new_HSP_HSPmanager(hspm,target,query_start,target_start,length,score)
 *
 * Descrip:    Adds a new HSP when all info is known
 *
 *
 * Arg:                hspm [UNKN ] Undocumented argument [HSPmanager *]
 * Arg:              target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_start [UNKN ] Undocumented argument [int]
 * Arg:        target_start [UNKN ] Undocumented argument [int]
 * Arg:              length [UNKN ] Undocumented argument [int]
 * Arg:               score [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean Wise2_add_new_HSP_HSPmanager(HSPmanager * hspm,Sequence * target,int query_start,int target_start,int length,int score);
#define add_new_HSP_HSPmanager Wise2_add_new_HSP_HSPmanager


/* Function:  free_ghash_HSPsets(key,value,user_data)
 *
 * Descrip:    Frees the HSPsets
 *
 *
 * Arg:              key [UNKN ] Undocumented argument [gpointer]
 * Arg:            value [UNKN ] Undocumented argument [gpointer]
 * Arg:        user_data [UNKN ] Undocumented argument [gpointer]
 *
 */
void Wise2_free_ghash_HSPsets(gpointer key,gpointer value,gpointer user_data);
#define free_ghash_HSPsets Wise2_free_ghash_HSPsets


/* Function:  free_HSPmanager(h)
 *
 * Descrip:    Frees the HSPmanager
 *
 *
 * Arg:        h [UNKN ] Undocumented argument [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
HSPmanager * Wise2_free_HSPmanager(HSPmanager * h);
#define free_HSPmanager Wise2_free_HSPmanager


/* Function:  on_HSP(test,query_pos,target_pos)
 *
 * Descrip:    tests whether this point is on this test HSP
 *
 *
 * Arg:              test [UNKN ] Undocumented argument [HSP *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean Wise2_on_HSP(HSP * test,int query_pos,int target_pos);
#define on_HSP Wise2_on_HSP


/* Function:  new_HSP_QuerySeqHSP(cache,query,target,query_pos,target_pos,mat,drop_off)
 *
 * Descrip:    builds a new HSP for these sequences
 *
 *
 * Arg:             cache [UNKN ] Undocumented argument [HSPCache *]
 * Arg:             query [UNKN ] Undocumented argument [QuerySeqHSP *]
 * Arg:            target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 * Arg:               mat [UNKN ] Undocumented argument [CompMat *]
 * Arg:          drop_off [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * Wise2_new_HSP_QuerySeqHSP(HSPCache * cache,QuerySeqHSP * query,Sequence * target,int query_pos,int target_pos,CompMat * mat,int drop_off);
#define new_HSP_QuerySeqHSP Wise2_new_HSP_QuerySeqHSP


/* Function:  new_HSP(cache,query,target,query_pos,target_pos,mat,drop_off)
 *
 * Descrip:    builds a new HSP for these sequences
 *
 *
 * Arg:             cache [UNKN ] Undocumented argument [HSPCache *]
 * Arg:             query [UNKN ] Undocumented argument [Sequence *]
 * Arg:            target [UNKN ] Undocumented argument [Sequence *]
 * Arg:         query_pos [UNKN ] Undocumented argument [int]
 * Arg:        target_pos [UNKN ] Undocumented argument [int]
 * Arg:               mat [UNKN ] Undocumented argument [CompMat *]
 * Arg:          drop_off [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * Wise2_new_HSP(HSPCache * cache,Sequence * query,Sequence * target,int query_pos,int target_pos,CompMat * mat,int drop_off);
#define new_HSP Wise2_new_HSP


/* Function:  HSP_alloc_cache(hspc)
 *
 * Descrip:    Returns new HSP, using cache if needed
 *
 *
 * Arg:        hspc [UNKN ] Undocumented argument [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * Wise2_HSP_alloc_cache(HSPCache * hspc);
#define HSP_alloc_cache Wise2_HSP_alloc_cache


/* Function:  free_HSP_cache(cache,hsp)
 *
 * Descrip:    Places HSP back into cache, freeing if necessary
 *
 *
 * Arg:        cache [UNKN ] Undocumented argument [HSPCache *]
 * Arg:          hsp [UNKN ] Undocumented argument [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * Wise2_free_HSP_cache(HSPCache * cache,HSP * hsp);
#define free_HSP_cache Wise2_free_HSP_cache


/* Function:  new_HSPCache(maxsize)
 *
 * Descrip:    Makes a new cache
 *
 *
 * Arg:        maxsize [UNKN ] Undocumented argument [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * Wise2_new_HSPCache(int maxsize);
#define new_HSPCache Wise2_new_HSPCache


/* Function:  show_HSPset(s,ofp)
 *
 * Descrip:    Shows a HSP set
 *
 *
 * Arg:          s [UNKN ] Undocumented argument [HSPset *]
 * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
 *
 */
void Wise2_show_HSPset(HSPset * s,FILE * ofp);
#define show_HSPset Wise2_show_HSPset


/* Function:  show_HSP(hsp,linelength,out)
 *
 * Descrip:    Shows an HSP
 *
 *
 * Arg:               hsp [UNKN ] Undocumented argument [HSP *]
 * Arg:        linelength [UNKN ] Undocumented argument [int]
 * Arg:               out [UNKN ] Undocumented argument [FILE *]
 *
 */
void Wise2_show_HSP(HSP * hsp,int linelength,FILE * out);
#define show_HSP Wise2_show_HSP


/* Function:  new_QuerySeqHSP(seq,mat)
 *
 * Descrip:    Builds a new QuerySeqHSP from Sequence and Matrix
 *
 *
 * Arg:        seq [UNKN ] Undocumented argument [Sequence *]
 * Arg:        mat [UNKN ] Undocumented argument [CompMat *]
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
QuerySeqHSP * Wise2_new_QuerySeqHSP(Sequence * seq,CompMat * mat);
#define new_QuerySeqHSP Wise2_new_QuerySeqHSP


/* Function:  hard_link_HSP(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * Wise2_hard_link_HSP(HSP * obj);
#define hard_link_HSP Wise2_hard_link_HSP


/* Function:  HSP_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * Wise2_HSP_alloc(void);
#define HSP_alloc Wise2_HSP_alloc


/* Function:  free_HSP(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [HSP *]
 *
 */
HSP * Wise2_free_HSP(HSP * obj);
#define free_HSP Wise2_free_HSP


/* Function:  add_HSPCache(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [HSPCache *]
 * Arg:        add [OWNER] Object to add to the list [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean Wise2_add_HSPCache(HSPCache * obj,HSP * add);
#define add_HSPCache Wise2_add_HSPCache


/* Function:  flush_HSPCache(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int Wise2_flush_HSPCache(HSPCache * obj);
#define flush_HSPCache Wise2_flush_HSPCache


/* Function:  HSPCache_alloc_std(void)
 *
 * Descrip:    Equivalent to HSPCache_alloc_len(HSPCacheLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * Wise2_HSPCache_alloc_std(void);
#define HSPCache_alloc_std Wise2_HSPCache_alloc_std


/* Function:  HSPCache_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * Wise2_HSPCache_alloc_len(int len);
#define HSPCache_alloc_len Wise2_HSPCache_alloc_len


/* Function:  hard_link_HSPCache(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * Wise2_hard_link_HSPCache(HSPCache * obj);
#define hard_link_HSPCache Wise2_hard_link_HSPCache


/* Function:  HSPCache_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * Wise2_HSPCache_alloc(void);
#define HSPCache_alloc Wise2_HSPCache_alloc


/* Function:  free_HSPCache(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [HSPCache *]
 *
 * Return [UNKN ]  Undocumented return value [HSPCache *]
 *
 */
HSPCache * Wise2_free_HSPCache(HSPCache * obj);
#define free_HSPCache Wise2_free_HSPCache


/* Function:  add_HSPset(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [HSPset *]
 * Arg:        add [OWNER] Object to add to the list [HSP *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean Wise2_add_HSPset(HSPset * obj,HSP * add);
#define add_HSPset Wise2_add_HSPset


/* Function:  flush_HSPset(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int Wise2_flush_HSPset(HSPset * obj);
#define flush_HSPset Wise2_flush_HSPset


/* Function:  HSPset_alloc_std(void)
 *
 * Descrip:    Equivalent to HSPset_alloc_len(HSPsetLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * Wise2_HSPset_alloc_std(void);
#define HSPset_alloc_std Wise2_HSPset_alloc_std


/* Function:  HSPset_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * Wise2_HSPset_alloc_len(int len);
#define HSPset_alloc_len Wise2_HSPset_alloc_len


/* Function:  hard_link_HSPset(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * Wise2_hard_link_HSPset(HSPset * obj);
#define hard_link_HSPset Wise2_hard_link_HSPset


/* Function:  HSPset_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * Wise2_HSPset_alloc(void);
#define HSPset_alloc Wise2_HSPset_alloc


/* Function:  free_HSPset(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [HSPset *]
 *
 */
HSPset * Wise2_free_HSPset(HSPset * obj);
#define free_HSPset Wise2_free_HSPset


/* Function:  hard_link_TopScoreManager(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TopScoreManager *]
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
TopScoreManager * Wise2_hard_link_TopScoreManager(TopScoreManager * obj);
#define hard_link_TopScoreManager Wise2_hard_link_TopScoreManager


/* Function:  TopScoreManager_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
TopScoreManager * Wise2_TopScoreManager_alloc(void);
#define TopScoreManager_alloc Wise2_TopScoreManager_alloc


/* Function:  free_TopScoreManager(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TopScoreManager *]
 *
 * Return [UNKN ]  Undocumented return value [TopScoreManager *]
 *
 */
TopScoreManager * Wise2_free_TopScoreManager(TopScoreManager * obj);
#define free_TopScoreManager Wise2_free_TopScoreManager


/* Function:  hard_link_QuerySeqHSP(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [QuerySeqHSP *]
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
QuerySeqHSP * Wise2_hard_link_QuerySeqHSP(QuerySeqHSP * obj);
#define hard_link_QuerySeqHSP Wise2_hard_link_QuerySeqHSP


/* Function:  QuerySeqHSP_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
QuerySeqHSP * Wise2_QuerySeqHSP_alloc(void);
#define QuerySeqHSP_alloc Wise2_QuerySeqHSP_alloc


/* Function:  free_QuerySeqHSP(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [QuerySeqHSP *]
 *
 * Return [UNKN ]  Undocumented return value [QuerySeqHSP *]
 *
 */
QuerySeqHSP * Wise2_free_QuerySeqHSP(QuerySeqHSP * obj);
#define free_QuerySeqHSP Wise2_free_QuerySeqHSP


/* Function:  hard_link_HSPmanager(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [HSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
HSPmanager * Wise2_hard_link_HSPmanager(HSPmanager * obj);
#define hard_link_HSPmanager Wise2_hard_link_HSPmanager


/* Function:  HSPmanager_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [HSPmanager *]
 *
 */
HSPmanager * Wise2_HSPmanager_alloc(void);
#define HSPmanager_alloc Wise2_HSPmanager_alloc


/* Function:  add_LinearHSPmanager(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [LinearHSPmanager *]
 * Arg:        add [OWNER] Object to add to the list [HSPset *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean Wise2_add_LinearHSPmanager(LinearHSPmanager * obj,HSPset * add);
#define add_LinearHSPmanager Wise2_add_LinearHSPmanager


/* Function:  flush_LinearHSPmanager(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [LinearHSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int Wise2_flush_LinearHSPmanager(LinearHSPmanager * obj);
#define flush_LinearHSPmanager Wise2_flush_LinearHSPmanager


/* Function:  LinearHSPmanager_alloc_std(void)
 *
 * Descrip:    Equivalent to LinearHSPmanager_alloc_len(LinearHSPmanagerLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_LinearHSPmanager_alloc_std(void);
#define LinearHSPmanager_alloc_std Wise2_LinearHSPmanager_alloc_std


/* Function:  LinearHSPmanager_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_LinearHSPmanager_alloc_len(int len);
#define LinearHSPmanager_alloc_len Wise2_LinearHSPmanager_alloc_len


/* Function:  hard_link_LinearHSPmanager(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [LinearHSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_hard_link_LinearHSPmanager(LinearHSPmanager * obj);
#define hard_link_LinearHSPmanager Wise2_hard_link_LinearHSPmanager


/* Function:  LinearHSPmanager_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_LinearHSPmanager_alloc(void);
#define LinearHSPmanager_alloc Wise2_LinearHSPmanager_alloc


/* Function:  free_LinearHSPmanager(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [LinearHSPmanager *]
 *
 * Return [UNKN ]  Undocumented return value [LinearHSPmanager *]
 *
 */
LinearHSPmanager * Wise2_free_LinearHSPmanager(LinearHSPmanager * obj);
#define free_LinearHSPmanager Wise2_free_LinearHSPmanager


  /* Unplaced functions */
  /* There has been no indication of the use of these functions */


    /***************************************************/
    /* Internal functions                              */
    /* you are not expected to have to call these      */
    /***************************************************/
int Wise2_compare_HSP_score(HSP * one,HSP * two);
#define compare_HSP_score Wise2_compare_HSP_score
void Wise2_swap_HSPCache(HSP ** list,int i,int j) ;
#define swap_HSPCache Wise2_swap_HSPCache
void Wise2_qsort_HSPCache(HSP ** list,int left,int right,int (*comp)(HSP * ,HSP * ));
#define qsort_HSPCache Wise2_qsort_HSPCache
void Wise2_sort_HSPCache(HSPCache * obj,int (*comp)(HSP *, HSP *));
#define sort_HSPCache Wise2_sort_HSPCache
boolean Wise2_expand_HSPCache(HSPCache * obj,int len);
#define expand_HSPCache Wise2_expand_HSPCache
void Wise2_swap_HSPset(HSP ** list,int i,int j) ;
#define swap_HSPset Wise2_swap_HSPset
void Wise2_qsort_HSPset(HSP ** list,int left,int right,int (*comp)(HSP * ,HSP * ));
#define qsort_HSPset Wise2_qsort_HSPset
void Wise2_sort_HSPset(HSPset * obj,int (*comp)(HSP *, HSP *));
#define sort_HSPset Wise2_sort_HSPset
boolean Wise2_expand_HSPset(HSPset * obj,int len);
#define expand_HSPset Wise2_expand_HSPset
void Wise2_swap_LinearHSPmanager(HSPset ** list,int i,int j) ;
#define swap_LinearHSPmanager Wise2_swap_LinearHSPmanager
void Wise2_qsort_LinearHSPmanager(HSPset ** list,int left,int right,int (*comp)(HSPset * ,HSPset * ));
#define qsort_LinearHSPmanager Wise2_qsort_LinearHSPmanager
void Wise2_sort_LinearHSPmanager(LinearHSPmanager * obj,int (*comp)(HSPset *, HSPset *));
#define sort_LinearHSPmanager Wise2_sort_LinearHSPmanager
boolean Wise2_expand_LinearHSPmanager(LinearHSPmanager * obj,int len);
#define expand_LinearHSPmanager Wise2_expand_LinearHSPmanager

#ifdef _cplusplus
}
#endif

#endif
