#ifdef _cplusplus
extern "C" {
#endif
#include "transfactor.h"

# line 42 "transfactor.dy"
void show_help_TransFactorMatchPara(FILE * ofp)
{
  fprintf(ofp,"TransFactor Match Parameters\n");
  fprintf(ofp,"   -tfm_cutoff   bits cutoff for real matches, default 7.5\n");
}

# line 48 "transfactor.dy"
void show_help_TransFactorBuildPara(FILE * ofp)
{
  fprintf(ofp,"TransFactor Build Parameters\n");
  fprintf(ofp,"   -tfb_pseudo   simple pseudo count, default 0.5\n");
  fprintf(ofp,"   -[no]tfb_warn warn on small sequence number [default yes]\n");
}

# line 55 "transfactor.dy"
TransFactorMatchPara * new_TransFactorMatchPara_from_argv(int * argc,char ** argv)
{
  TransFactorMatchPara * out;

  out = TransFactorMatchPara_alloc();

  strip_out_float_argument(argc,argv,"tfm_cutoff",&out->min_bits);

  return out;
}

# line 66 "transfactor.dy"
boolean build_pwm_TransFactor(TransFactor * tf,TransFactorBuildPara * p)
{
  assert(tf);
  assert(tf->seed);
  assert(p);

  tf->pwm = pwmDNA_from_SeqAlign(tf->seed,p->pseudo_count);

  fold_randommodel_pwmDNA(tf->pwm,p->rnd_dna);

  return TRUE;
}

# line 79 "transfactor.dy"
TransFactorMatchSet * calculate_TransFactorMatchSet(Sequence * seq,TransFactorSet * tfs,TransFactorMatchPara * p)
{
  int i;
  int j;
  int k;
  int len;
  double prob;
  double cutoff;
  TransFactorMatchSet * out;
  TransFactorMatch * m;

  assert(seq);
  assert(tfs);
  assert(p);

  cutoff = Bits2Probability(p->min_bits);

  out = TransFactorMatchSet_alloc_std();

  for(i=0;i<tfs->len;i++) {
    len = seq->len - tfs->factor[i]->pwm->len;
    for(j=0;j<len;j++) {
      prob = 1.0;
      for(k=0;k<tfs->factor[i]->pwm->len;k++) {
	prob *= tfs->factor[i]->pwm->pos[k]->emit[base_from_char(seq->seq[j+k])];
      }
      if( prob > cutoff ) {
	m = TransFactorMatch_alloc();
	m->start = j;
	m->bit_score = Score2Bits(Probability2Score(prob));
	m->factor = hard_link_TransFactor(tfs->factor[i]);
	add_TransFactorMatchSet(out,m);
      }
    }
  }

  return out;

}

# line 119 "transfactor.dy"
void write_TransFactorSet(TransFactorSet * tfs,FILE * ofp)
{
  int i;

  for(i=0;i<tfs->len;i++) {
    write_TransFactor(tfs->factor[i],ofp);
  }

}

# line 129 "transfactor.dy"
void write_TransFactor(TransFactor * tf,FILE * ofp)
{
  fprintf(ofp,"factor %s\n",tf->name);
  write_selex_SeqAlign(tf->seed,30,100,ofp);
  fprintf(ofp,"//\n");
  fprintf(ofp,"end factor\n");
}

# line 137 "transfactor.dy"
TransFactorSet * read_TransFactorSet(FILE * ifp)
{
  char buffer[MAXLINE];
  TransFactorSet * out;
  TransFactor * temp;

  out = TransFactorSet_alloc_std();
  
  while( fgets(buffer,MAXLINE,ifp) ) {
    if( buffer[0] == '#' ) {
      continue;
    }
    if( strstartcmp(buffer,"factor") == 0 ) {
      temp = read_TransFactor(buffer,ifp);
      if( temp == NULL ) {
	warn("No transfactor; skipping; parsing may have fallen down");
	continue;
      } else {
	add_TransFactorSet(out,temp);
      }
    }
    if( isalpha(buffer[0]) ) {
      warn("could not interpret in reading transfactorset %s",buffer);
    }
     
  }

  return out;


}


# line 170 "transfactor.dy"
TransFactor * read_TransFactor(char * line,FILE * ifp)
{
  char buffer[MAXLINE];
  char * name;
  char * t;
  TransFactor * out;
  SeqAlign * temp;

  if( strstartcmp(line,"factor") != 0 ) {
    warn("Passed in a line [%s] which has no factor to the factor reader. Not good!",line);
    return NULL;
  }


  name = line + strlen("factor")+1;
  for(;isspace(*name);name++)
    ;

  if( *name == '\0' ) {
    warn("No name in factor - which is bad! %s\n",line);
    return NULL;
  }
  for(t=name;*t && !isspace(*t);t++)
    ;
  *t = '\0';

  while( fgets(buffer,MAXLINE,ifp) != NULL ) {
    if( buffer[0] == '#' ) {
      continue;
    }

    if( strstartcmp(buffer,"seed") == 0 ) {
      temp = read_selex_SeqAlign(ifp);

      if( temp == NULL ) {
	warn("No sequence alignment for this factor. Skipping");
	return NULL;
      }
      continue;
    }
    if( strstartcmp(buffer,"end") == 0 ) {
      break;
    }
    if( isalpha(buffer[0]) ) {
      warn("Uninterpretable line in transfactor %s",buffer);
    }

    
  }

  if( temp == NULL ) {
    warn("No seed alignment read. Impossible for %s",name);
    return NULL;
  }

  out = TransFactor_alloc();
  out->seed = temp;
  out->name = stringalloc(name);
  
  return out;
}

# line 204 "transfactor.c"
/* Function:  hard_link_TransFactor(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TransFactor *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactor *]
 *
 */
TransFactor * hard_link_TransFactor(TransFactor * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a TransFactor object: passed a NULL object"); 
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  TransFactor_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactor *]
 *
 */
TransFactor * TransFactor_alloc(void) 
{
    TransFactor * out;  /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(TransFactor *) ckalloc (sizeof(TransFactor))) == NULL)  {  
      warn("TransFactor_alloc failed "); 
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->name = NULL;    
    out->seed = NULL;    
    out->pwm = NULL; 
    out->full = NULL;    


    return out;  
}    


/* Function:  free_TransFactor(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TransFactor *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactor *]
 *
 */
TransFactor * free_TransFactor(TransFactor * obj) 
{
    int return_early = 0;    


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a TransFactor obj. Should be trappable");   
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->name != NULL)   
      ckfree(obj->name);     
    if( obj->seed != NULL)   
      free_SeqAlign(obj->seed);  
    if( obj->pwm != NULL)    
      free_pwmDNA(obj->pwm);     
    if( obj->full != NULL)   
      free_SeqAlign(obj->full);  


    ckfree(obj); 
    return NULL; 
}    


/* Function:  swap_TransFactorSet(list,i,j)
 *
 * Descrip:    swap function: an internal for qsort_TransFactorSet
 *             swaps two positions in the array
 *
 *
 * Arg:        list [UNKN ] List of structures to swap in [TransFactor **]
 * Arg:           i [UNKN ] swap position [int]
 * Arg:           j [UNKN ] swap position [int]
 *
 */
/* swap function for qsort function */ 
void swap_TransFactorSet(TransFactor ** list,int i,int j)  
{
    TransFactor * temp;  
    temp=list[i];    
    list[i]=list[j]; 
    list[j]=temp;    
}    


/* Function:  qsort_TransFactorSet(list,left,right,comp)
 *
 * Descrip:    qsort - lifted from K&R 
 *             sorts the array using quicksort
 *             Probably much better to call sort_TransFactorSet which sorts from start to end
 *
 *
 * Arg:         list [UNKN ] List of structures to swap in [TransFactor **]
 * Arg:         left [UNKN ] left position [int]
 * Arg:        right [UNKN ] right position [int]
 * Arg:         comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void qsort_TransFactorSet(TransFactor ** list,int left,int right,int (*comp)(TransFactor * ,TransFactor * )) 
{
    int i,last;  
    if( left >= right )  
      return;    


    swap_TransFactorSet(list,left,(left+right)/2);   
    last = left; 
    for ( i=left+1; i <= right;i++)  {  
      if( (*comp)(list[i],list[left]) < 0)   
        swap_TransFactorSet (list,++last,i); 
      }  
    swap_TransFactorSet (list,left,last);    
    qsort_TransFactorSet(list,left,last-1,comp); 
    qsort_TransFactorSet(list,last+1,right,comp);    
}    


/* Function:  sort_TransFactorSet(obj,comp)
 *
 * Descrip:    sorts from start to end using comp 
 *             sorts the array using quicksort by calling qsort_TransFactorSet
 *
 *
 * Arg:         obj [UNKN ] Object containing list [TransFactorSet *]
 * Arg:        comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void sort_TransFactorSet(TransFactorSet * obj,int (*comp)(TransFactor *, TransFactor *)) 
{
    qsort_TransFactorSet(obj->factor,0,obj->len-1,comp); 
    return;  
}    


/* Function:  expand_TransFactorSet(obj,len)
 *
 * Descrip:    Really an internal function for add_TransFactorSet
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [TransFactorSet *]
 * Arg:        len [UNKN ] Length to add one [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean expand_TransFactorSet(TransFactorSet * obj,int len) 
{


    if( obj->maxlen > obj->len )     {  
      warn("expand_TransFactorSet called with no need"); 
      return TRUE;   
      }  


    if( (obj->factor = (TransFactor ** ) ckrealloc (obj->factor,sizeof(TransFactor *)*len)) == NULL)     {  
      warn("ckrealloc failed for expand_TransFactorSet, returning FALSE");   
      return FALSE;  
      }  
    obj->maxlen = len;   
    return TRUE; 
}    


/* Function:  add_TransFactorSet(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [TransFactorSet *]
 * Arg:        add [OWNER] Object to add to the list [TransFactor *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
/* will expand function if necessary */ 
boolean add_TransFactorSet(TransFactorSet * obj,TransFactor * add) 
{
    if( obj->len >= obj->maxlen) {  
      if( expand_TransFactorSet(obj,obj->len + TransFactorSetLISTLENGTH) == FALSE)   
        return FALSE;    
      }  


    obj->factor[obj->len++]=add; 
    return TRUE; 
}    


/* Function:  flush_TransFactorSet(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [TransFactorSet *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int flush_TransFactorSet(TransFactorSet * obj) 
{
    int i;   


    for(i=0;i<obj->len;i++)  { /*for i over list length*/ 
      if( obj->factor[i] != NULL)    {  
        free_TransFactor(obj->factor[i]);    
        obj->factor[i] = NULL;   
        }  
      } /* end of for i over list length */ 


    obj->len = 0;    
    return i;    
}    


/* Function:  TransFactorSet_alloc_std(void)
 *
 * Descrip:    Equivalent to TransFactorSet_alloc_len(TransFactorSetLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactorSet *]
 *
 */
TransFactorSet * TransFactorSet_alloc_std(void) 
{
    return TransFactorSet_alloc_len(TransFactorSetLISTLENGTH);   
}    


/* Function:  TransFactorSet_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorSet *]
 *
 */
TransFactorSet * TransFactorSet_alloc_len(int len) 
{
    TransFactorSet * out;   /* out is exported at the end of function */ 


    /* Call alloc function: return NULL if NULL */ 
    /* Warning message alread in alloc function */ 
    if((out = TransFactorSet_alloc()) == NULL)   
      return NULL;   


    /* Calling ckcalloc for list elements */ 
    if((out->factor = (TransFactor ** ) ckcalloc (len,sizeof(TransFactor *))) == NULL)   {  
      warn("Warning, ckcalloc failed in TransFactorSet_alloc_len");  
      return NULL;   
      }  
    out->len = 0;    
    out->maxlen = len;   


    return out;  
}    


/* Function:  hard_link_TransFactorSet(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TransFactorSet *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorSet *]
 *
 */
TransFactorSet * hard_link_TransFactorSet(TransFactorSet * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a TransFactorSet object: passed a NULL object");  
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  TransFactorSet_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactorSet *]
 *
 */
TransFactorSet * TransFactorSet_alloc(void) 
{
    TransFactorSet * out;   /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(TransFactorSet *) ckalloc (sizeof(TransFactorSet))) == NULL)    {  
      warn("TransFactorSet_alloc failed ");  
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->factor = NULL;  
    out->len = out->maxlen = 0;  


    return out;  
}    


/* Function:  free_TransFactorSet(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TransFactorSet *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorSet *]
 *
 */
TransFactorSet * free_TransFactorSet(TransFactorSet * obj) 
{
    int return_early = 0;    
    int i;   


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a TransFactorSet obj. Should be trappable");    
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->factor != NULL) {  
      for(i=0;i<obj->len;i++)    {  
        if( obj->factor[i] != NULL)  
          free_TransFactor(obj->factor[i]);  
        }  
      ckfree(obj->factor);   
      }  


    ckfree(obj); 
    return NULL; 
}    


/* Function:  hard_link_TransFactorMatch(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TransFactorMatch *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatch *]
 *
 */
TransFactorMatch * hard_link_TransFactorMatch(TransFactorMatch * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a TransFactorMatch object: passed a NULL object");    
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  TransFactorMatch_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatch *]
 *
 */
TransFactorMatch * TransFactorMatch_alloc(void) 
{
    TransFactorMatch * out; /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(TransFactorMatch *) ckalloc (sizeof(TransFactorMatch))) == NULL)    {  
      warn("TransFactorMatch_alloc failed ");    
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->start = 0;  
    out->bit_score = 0;  
    out->factor = NULL;  


    return out;  
}    


/* Function:  free_TransFactorMatch(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TransFactorMatch *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatch *]
 *
 */
TransFactorMatch * free_TransFactorMatch(TransFactorMatch * obj) 
{
    int return_early = 0;    


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a TransFactorMatch obj. Should be trappable");  
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->factor != NULL) 
      free_TransFactor(obj->factor);     


    ckfree(obj); 
    return NULL; 
}    


/* Function:  swap_TransFactorMatchSet(list,i,j)
 *
 * Descrip:    swap function: an internal for qsort_TransFactorMatchSet
 *             swaps two positions in the array
 *
 *
 * Arg:        list [UNKN ] List of structures to swap in [TransFactorMatch **]
 * Arg:           i [UNKN ] swap position [int]
 * Arg:           j [UNKN ] swap position [int]
 *
 */
/* swap function for qsort function */ 
void swap_TransFactorMatchSet(TransFactorMatch ** list,int i,int j)  
{
    TransFactorMatch * temp; 
    temp=list[i];    
    list[i]=list[j]; 
    list[j]=temp;    
}    


/* Function:  qsort_TransFactorMatchSet(list,left,right,comp)
 *
 * Descrip:    qsort - lifted from K&R 
 *             sorts the array using quicksort
 *             Probably much better to call sort_TransFactorMatchSet which sorts from start to end
 *
 *
 * Arg:         list [UNKN ] List of structures to swap in [TransFactorMatch **]
 * Arg:         left [UNKN ] left position [int]
 * Arg:        right [UNKN ] right position [int]
 * Arg:         comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void qsort_TransFactorMatchSet(TransFactorMatch ** list,int left,int right,int (*comp)(TransFactorMatch * ,TransFactorMatch * )) 
{
    int i,last;  
    if( left >= right )  
      return;    


    swap_TransFactorMatchSet(list,left,(left+right)/2);  
    last = left; 
    for ( i=left+1; i <= right;i++)  {  
      if( (*comp)(list[i],list[left]) < 0)   
        swap_TransFactorMatchSet (list,++last,i);    
      }  
    swap_TransFactorMatchSet (list,left,last);   
    qsort_TransFactorMatchSet(list,left,last-1,comp);    
    qsort_TransFactorMatchSet(list,last+1,right,comp);   
}    


/* Function:  sort_TransFactorMatchSet(obj,comp)
 *
 * Descrip:    sorts from start to end using comp 
 *             sorts the array using quicksort by calling qsort_TransFactorMatchSet
 *
 *
 * Arg:         obj [UNKN ] Object containing list [TransFactorMatchSet *]
 * Arg:        comp [FUNCP] Function which returns -1 or 1 to sort on [int (*comp]
 *
 */
void sort_TransFactorMatchSet(TransFactorMatchSet * obj,int (*comp)(TransFactorMatch *, TransFactorMatch *)) 
{
    qsort_TransFactorMatchSet(obj->match,0,obj->len-1,comp); 
    return;  
}    


/* Function:  expand_TransFactorMatchSet(obj,len)
 *
 * Descrip:    Really an internal function for add_TransFactorMatchSet
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [TransFactorMatchSet *]
 * Arg:        len [UNKN ] Length to add one [int]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
boolean expand_TransFactorMatchSet(TransFactorMatchSet * obj,int len) 
{


    if( obj->maxlen > obj->len )     {  
      warn("expand_TransFactorMatchSet called with no need");    
      return TRUE;   
      }  


    if( (obj->match = (TransFactorMatch ** ) ckrealloc (obj->match,sizeof(TransFactorMatch *)*len)) == NULL)     {  
      warn("ckrealloc failed for expand_TransFactorMatchSet, returning FALSE");  
      return FALSE;  
      }  
    obj->maxlen = len;   
    return TRUE; 
}    


/* Function:  add_TransFactorMatchSet(obj,add)
 *
 * Descrip:    Adds another object to the list. It will expand the list if necessary
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list [TransFactorMatchSet *]
 * Arg:        add [OWNER] Object to add to the list [TransFactorMatch *]
 *
 * Return [UNKN ]  Undocumented return value [boolean]
 *
 */
/* will expand function if necessary */ 
boolean add_TransFactorMatchSet(TransFactorMatchSet * obj,TransFactorMatch * add) 
{
    if( obj->len >= obj->maxlen) {  
      if( expand_TransFactorMatchSet(obj,obj->len + TransFactorMatchSetLISTLENGTH) == FALSE) 
        return FALSE;    
      }  


    obj->match[obj->len++]=add;  
    return TRUE; 
}    


/* Function:  flush_TransFactorMatchSet(obj)
 *
 * Descrip:    Frees the list elements, sets length to 0
 *             If you want to save some elements, use hard_link_xxx
 *             to protect them from being actually destroyed in the free
 *
 *
 * Arg:        obj [UNKN ] Object which contains the list  [TransFactorMatchSet *]
 *
 * Return [UNKN ]  Undocumented return value [int]
 *
 */
int flush_TransFactorMatchSet(TransFactorMatchSet * obj) 
{
    int i;   


    for(i=0;i<obj->len;i++)  { /*for i over list length*/ 
      if( obj->match[i] != NULL) {  
        free_TransFactorMatch(obj->match[i]);    
        obj->match[i] = NULL;    
        }  
      } /* end of for i over list length */ 


    obj->len = 0;    
    return i;    
}    


/* Function:  TransFactorMatchSet_alloc_std(void)
 *
 * Descrip:    Equivalent to TransFactorMatchSet_alloc_len(TransFactorMatchSetLISTLENGTH)
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchSet *]
 *
 */
TransFactorMatchSet * TransFactorMatchSet_alloc_std(void) 
{
    return TransFactorMatchSet_alloc_len(TransFactorMatchSetLISTLENGTH); 
}    


/* Function:  TransFactorMatchSet_alloc_len(len)
 *
 * Descrip:    Allocates len length to all lists
 *
 *
 * Arg:        len [UNKN ] Length of lists to allocate [int]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchSet *]
 *
 */
TransFactorMatchSet * TransFactorMatchSet_alloc_len(int len) 
{
    TransFactorMatchSet * out;  /* out is exported at the end of function */ 


    /* Call alloc function: return NULL if NULL */ 
    /* Warning message alread in alloc function */ 
    if((out = TransFactorMatchSet_alloc()) == NULL)  
      return NULL;   


    /* Calling ckcalloc for list elements */ 
    if((out->match = (TransFactorMatch ** ) ckcalloc (len,sizeof(TransFactorMatch *))) == NULL)  {  
      warn("Warning, ckcalloc failed in TransFactorMatchSet_alloc_len"); 
      return NULL;   
      }  
    out->len = 0;    
    out->maxlen = len;   


    return out;  
}    


/* Function:  hard_link_TransFactorMatchSet(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TransFactorMatchSet *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchSet *]
 *
 */
TransFactorMatchSet * hard_link_TransFactorMatchSet(TransFactorMatchSet * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a TransFactorMatchSet object: passed a NULL object"); 
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  TransFactorMatchSet_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchSet *]
 *
 */
TransFactorMatchSet * TransFactorMatchSet_alloc(void) 
{
    TransFactorMatchSet * out;  /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(TransFactorMatchSet *) ckalloc (sizeof(TransFactorMatchSet))) == NULL)  {  
      warn("TransFactorMatchSet_alloc failed "); 
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->match = NULL;   
    out->len = out->maxlen = 0;  


    return out;  
}    


/* Function:  free_TransFactorMatchSet(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TransFactorMatchSet *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchSet *]
 *
 */
TransFactorMatchSet * free_TransFactorMatchSet(TransFactorMatchSet * obj) 
{
    int return_early = 0;    
    int i;   


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a TransFactorMatchSet obj. Should be trappable");   
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->match != NULL)  {  
      for(i=0;i<obj->len;i++)    {  
        if( obj->match[i] != NULL)   
          free_TransFactorMatch(obj->match[i]);  
        }  
      ckfree(obj->match);    
      }  


    ckfree(obj); 
    return NULL; 
}    


/* Function:  hard_link_TransFactorBuildPara(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TransFactorBuildPara *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorBuildPara *]
 *
 */
TransFactorBuildPara * hard_link_TransFactorBuildPara(TransFactorBuildPara * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a TransFactorBuildPara object: passed a NULL object");    
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  TransFactorBuildPara_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactorBuildPara *]
 *
 */
TransFactorBuildPara * TransFactorBuildPara_alloc(void) 
{
    TransFactorBuildPara * out; /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(TransFactorBuildPara *) ckalloc (sizeof(TransFactorBuildPara))) == NULL)    {  
      warn("TransFactorBuildPara_alloc failed ");    
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->rnd_dna = NULL; 
    out->pseudo_count = 0.5; 
    out->warn_on_small_seq = TRUE;   


    return out;  
}    


/* Function:  free_TransFactorBuildPara(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TransFactorBuildPara *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorBuildPara *]
 *
 */
TransFactorBuildPara * free_TransFactorBuildPara(TransFactorBuildPara * obj) 
{
    int return_early = 0;    


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a TransFactorBuildPara obj. Should be trappable");  
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   
    if( obj->rnd_dna != NULL)    
      free_RandomModelDNA(obj->rnd_dna);     


    ckfree(obj); 
    return NULL; 
}    


/* Function:  hard_link_TransFactorMatchPara(obj)
 *
 * Descrip:    Bumps up the reference count of the object
 *             Meaning that multiple pointers can 'own' it
 *
 *
 * Arg:        obj [UNKN ] Object to be hard linked [TransFactorMatchPara *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchPara *]
 *
 */
TransFactorMatchPara * hard_link_TransFactorMatchPara(TransFactorMatchPara * obj) 
{
    if( obj == NULL )    {  
      warn("Trying to hard link to a TransFactorMatchPara object: passed a NULL object");    
      return NULL;   
      }  
    obj->dynamite_hard_link++;   
    return obj;  
}    


/* Function:  TransFactorMatchPara_alloc(void)
 *
 * Descrip:    Allocates structure: assigns defaults if given 
 *
 *
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchPara *]
 *
 */
TransFactorMatchPara * TransFactorMatchPara_alloc(void) 
{
    TransFactorMatchPara * out; /* out is exported at end of function */ 


    /* call ckalloc and see if NULL */ 
    if((out=(TransFactorMatchPara *) ckalloc (sizeof(TransFactorMatchPara))) == NULL)    {  
      warn("TransFactorMatchPara_alloc failed ");    
      return NULL;  /* calling function should respond! */ 
      }  
    out->dynamite_hard_link = 1; 
#ifdef PTHREAD   
    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
#endif   
    out->min_bits = 7.5; 


    return out;  
}    


/* Function:  free_TransFactorMatchPara(obj)
 *
 * Descrip:    Free Function: removes the memory held by obj
 *             Will chain up to owned members and clear all lists
 *
 *
 * Arg:        obj [UNKN ] Object that is free'd [TransFactorMatchPara *]
 *
 * Return [UNKN ]  Undocumented return value [TransFactorMatchPara *]
 *
 */
TransFactorMatchPara * free_TransFactorMatchPara(TransFactorMatchPara * obj) 
{
    int return_early = 0;    


    if( obj == NULL) {  
      warn("Attempting to free a NULL pointer to a TransFactorMatchPara obj. Should be trappable");  
      return NULL;   
      }  


#ifdef PTHREAD   
    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
#endif   
    if( obj->dynamite_hard_link > 1)     {  
      return_early = 1;  
      obj->dynamite_hard_link--; 
      }  
#ifdef PTHREAD   
    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
#endif   
    if( return_early == 1)   
      return NULL;   


    ckfree(obj); 
    return NULL; 
}    



#ifdef _cplusplus
}
#endif
